import os
import argparse

from .launch import handle
from .utils.utils import PATH


CURRENT_PATH = PATH

# Initalise function to call other functions based on arguments
def initalise(build=None, add=None, show_list=None, remove=None, purge=None, env=None, ls=False):
    
    
    if build:
        # Create new work environment
        filename = f'{CURRENT_PATH}\{build}'
        handle.build(filename, build)
    
    
    if add:
        # Add program to existing work environment
        filename = f'{CURRENT_PATH}\{add}'
        handle.add(filename)
        
    
    if show_list:
        # Show all programs in the environment
        filename = f'{CURRENT_PATH}\{show_list}'
        handle.show_lst(filename, show_list)
        
    
    if remove:
        # Delete program from given environment
        filename = f'{CURRENT_PATH}\{remove}'
        handle.remove_element(filename)
        
    
    if purge:
        # Remove the work environment
        filename = f'{CURRENT_PATH}\{purge}' + '.json'
        handle.purge(filename, purge)
    
    
    if env:
        # Run environment
        filename = f'{CURRENT_PATH}\{env}'
        handle.launch(filename, env)
        
    if ls:
        # List all availabel work environments
        handle.ls()
        
def main():
    # Parser arguments
    parser = argparse.ArgumentParser(description='Run environment')
    parser.add_argument('-build', '--build', help='Create new work environment/workspace', 
                         type=str, metavar='')
    parser.add_argument('-a', '--add', help='Add program to existing work environment', 
                        type=str, metavar='')
    parser.add_argument('-s', '--show', help='Show all program in given work environment', 
                        type=str, metavar='')
    parser.add_argument('-remove', '--remove', help='Remove program from given work environment', 
                        type=str, metavar='')
    parser.add_argument('-purge', '--purge', help='delete the work environment file', 
                        type=str, metavar='')
    parser.add_argument('-env', '--env', help='Run environment', 
                        type=str, metavar='')
    parser.add_argument('-ls', '--list', help='List all available work environments', 
                        action='store_true')
    args = parser.parse_args()
    
    initalise(build=args.build, add=args.add, 
              show_list=args.show, remove=args.remove, 
              purge=args.purge, env=args.env , ls=args.list)
    
    
if __name__ == '__main__':
    main()


import os
import subprocess

from .environment.environment import data
from .utils.utils import PATH, logo

CURRENT_PATH = PATH

class handle:
    """Logic for launch"""
    
    
    def add(filename):
        # Add to launch file
        name = input('Enter name of program to add this environment: ')
        path = input('Enter path of program executable: ')
        try:
            data.add_json(filename, name, path)
        except Exception as Error:
            print(Error)
            
            
    def build(filename, build):
        # Build launch file
        print(f'Launch file with name {build} will be created...')
        if os.path.exists(f'{filename}.json'):
            print('File already exist. Try another name')
        else:
            data.create_json(filename)
            
            
    def launch(filename, env):
        # launch function
        try:
            dic = data.read_json(filename)
            print(f'Launching workspace - {env}')
            logo()
            handle.run(dic)
        except FileNotFoundError:
            print(f'No Workspace named {env} found')
            handle.ls()
    
    
    def purge(filename, purge):
        # Delete launch file
        if os.path.exists(filename):
            verify = input(f'Confirm to delete {purge} environment [y/n]: ')
            if verify == 'y':
                os.remove(filename)
        else:
            print(f'No environment named {purge}.')
    
    
    def remove_element(filename):
        # Remove element from launch file 
        item = input('Enter program name to delete from environment: ')
        data.remove_element_json(filename, item)
    
    
    def show_lst(filename, show_list):
        # List all elements from launch file
        try:
            dic = data.read_json(filename)
            print(f'Workspace {show_list}')
            for _ in dic:
                print(f' {_}: {dic[_]}')
        except FileNotFoundError:
            print(f'No Workspace named {show_list} found')

    
    @staticmethod
    def ls():
        # List all launch environments
        print('Here is list of available workspace:')
        for _ in os.listdir(CURRENT_PATH):
            if _.endswith('.json'):
                print(' ',_.replace('.json',''))
            
    
    @staticmethod                
    def run(dic):
        # Run programs from launch file
        for _ in dic:
            try:
                subprocess.Popen(dic[_])
                print(f'Executed {_}')
            except Exception:
                print(f'Could not open {_}:{dic[_]}. Check if path/file_name is correct')


import json

class data:
    """Logic for launch file data """

    
    def read_json(filename):
        # Read json and return dictionary
        with open (filename + '.json', 'r') as f:
            dic = json.load(f)
            return dic

    
    def create_json(filename):
        # create json file
        dic = {}
        with open(filename + '.json', 'a+') as f:
            json.dump(dic, f)

    
    def add_json(filename, key, value):
        # Add program to json file
        with open (filename + '.json', 'r+') as f:
            dic = json.load(f)
            dic = dic.copy()
            dic.update({key:value})
            f.seek(0)
            json.dump(dic, f, indent=2)

    
    def remove_element_json(filename, item_key):
        # Remove program from json file
        with open (filename + '.json') as f:
            dic = json.load(f)
            dic = dic.copy()
            if item_key not in dic:
                print(f'Program {item_key} not found')
            del dic[item_key]
            
        with open (filename + '.json', 'w') as f:
            json.dump(dic, f)



from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.2'
DESCRIPTION = 'Launch all your programs and scripts with one command'

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
]

# Setting up
setup(
    name="launchenv",
    version=VERSION,
    author="Atharva Bhandvalkar",
    author_email="<atharv.bhandvalkar@gmail.com>",
    license='MIT',
    url='https://github.com/a-tharva/LaunchEnv',
    description=DESCRIPTION,
    long_description=long_description + '\n\n' + open('CHANGELOG.txt').read(),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=[],
    keywords=['python'],
    entry_points={
        'console_scripts': [
            'launch = launch.main:main',
        ],
    },
    classifiers=classifiers,
)


#import subprocess
#
#subprocess.Popen(f'C:\Windows\system32\calc.exe')
#
#
#
#
#
#
#
#\ is a escape character. You have three options.
#
#1) use /. This, as a bonus works for linux as well:
# 'D:/xxx/xxxx/asd/asd.exe'
# 
#2) escape the backslash
# 'D:\\xxx\\xxxx\\asd\\asd.exe'
# 
#3) use raw strings:
#r'D:\xxx\xxxx\asd\asd.exe'





#import json 
#
#with open('/src/environment/data.json', 'r+') as f:
#    lst = json.load(f)
#    print(lst.index(bee))


#import pyfiglet
#  
#result = pyfiglet.figlet_format("fort", font = "small"  )
#print(result)
